import _regeneratorRuntime from "@babel/runtime/regenerator";
import _asyncToGenerator from "@babel/runtime/helpers/esm/asyncToGenerator";
import _extends from "@babel/runtime/helpers/esm/extends";
import { useContext, useReducer, useRef } from 'react';
import pick from 'object.pick';
import { reducerUtils, createReducerCase } from './reducer-utils';
import { AppContext } from './app-context';

var CreateStoreHook = /*#__PURE__*/function () {
  function CreateStoreHook() {}

  var _proto = CreateStoreHook.prototype;

  _proto.writeGetUseReducerConfig = function writeGetUseReducerConfig(storeConfig) {
    var initStateMeta = storeConfig.initState,
        ref = storeConfig.ref;
    var useReducerConfig = {
      refKeys: Object.keys(ref)
    };
    var initStateMetaIsArray = Array.isArray(initStateMeta);

    if (initStateMetaIsArray) {
      useReducerConfig.initState = initStateMeta[0];
      useReducerConfig.init = initStateMeta[1];
      useReducerConfig.stateKeys = Object.keys(initStateMeta[0]);
    } else {
      useReducerConfig.initState = initStateMeta;

      useReducerConfig.init = function (state) {
        return state;
      };

      useReducerConfig.stateKeys = Object.keys(initStateMeta);
    }

    return useReducerConfig;
  };

  _proto.writeGetUseReducerConfigWithMembrane = function writeGetUseReducerConfigWithMembrane(storeConfig) {
    var superUseReducerConfig = this.writeGetUseReducerConfig(storeConfig);

    if (storeConfig.membrane) {
      var membraneUseReducerConfig = this.writeGetUseReducerConfig(storeConfig.membrane);
      return {
        stateKeys: [].concat(superUseReducerConfig.stateKeys, membraneUseReducerConfig.stateKeys),
        initState: _extends({}, superUseReducerConfig.initState, membraneUseReducerConfig.initState),
        init: function init(initArgs) {
          return membraneUseReducerConfig.init(superUseReducerConfig.init(initArgs));
        },
        refKeys: [].concat(superUseReducerConfig.refKeys, membraneUseReducerConfig.refKeys)
      };
    }

    return superUseReducerConfig;
  };

  _proto.writeGetControllerKeys = function writeGetControllerKeys(controller) {
    var controllerKeys = Object.keys(controller);
    controllerKeys.forEach(function (controllerKey) {
      var isNotStartWithON = !controllerKey.startsWith('on');

      if (isNotStartWithON) {
        throw new Error(controllerKey + " \u547D\u540D\u5FC5\u987B\u4EE5 on \u5F00\u5934, \u540D\u8BCD + \u52A8\u8BCD\u7ED3\u5C3E");
      }
    });
    return controllerKeys;
  };

  _proto.writeGetController = function writeGetController(_ref, contextProps, serviceBindContext) {
    var _this = this;

    var controller = _ref.controller;
    var controllerBindContext = {};
    var controllerKeys = this.writeGetControllerKeys(controller);
    console.log(controllerKeys, controller, 65);
    controllerKeys.forEach(function (controllerKey) {
      var controllerIsArray = Array.isArray(controller[controllerKey]);

      if (controllerIsArray) {
        controllerBindContext[controllerKey] = _this.writeControllerBindHandler(controller[controllerKey][1], contextProps, serviceBindContext);
      } else {
        controllerBindContext[controllerKey] = _this.writeControllerBindHandler(controller[controllerKey], contextProps, serviceBindContext);
      }
    });
    return controllerBindContext;
  };

  _proto.writeControllerBindHandler = function writeControllerBindHandler(target, contextProps, serviceBindContext) {
    return /*#__PURE__*/_asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee() {
      var _len,
          args,
          _key,
          res,
          _args = arguments;

      return _regeneratorRuntime.wrap(function _callee$(_context) {
        while (1) {
          switch (_context.prev = _context.next) {
            case 0:
              for (_len = _args.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
                args[_key] = _args[_key];
              }

              _context.next = 3;
              return target.call.apply(target, [Object.freeze({
                context: contextProps.context,
                state: contextProps.state,
                refs: contextProps.refs,
                rc: contextProps.rc,
                service: serviceBindContext,
                super: contextProps.superContext || null
              })].concat(args));

            case 3:
              res = _context.sent;
              return _context.abrupt("return", res);

            case 5:
            case "end":
              return _context.stop();
          }
        }
      }, _callee);
    }));
  };

  _proto.writeServiceBindHandler = function writeServiceBindHandler(target, contextProps, serviceBindContext) {
    return /*#__PURE__*/_asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee2() {
      var _len2,
          args,
          _key2,
          res,
          _args2 = arguments;

      return _regeneratorRuntime.wrap(function _callee2$(_context2) {
        while (1) {
          switch (_context2.prev = _context2.next) {
            case 0:
              for (_len2 = _args2.length, args = new Array(_len2), _key2 = 0; _key2 < _len2; _key2++) {
                args[_key2] = _args2[_key2];
              }

              _context2.next = 3;
              return target.call.apply(target, [Object.freeze({
                state: contextProps.state,
                refs: contextProps.refs,
                service: serviceBindContext,
                super: contextProps.superContext || null
              })].concat(args));

            case 3:
              res = _context2.sent;
              return _context2.abrupt("return", res);

            case 5:
            case "end":
              return _context2.stop();
          }
        }
      }, _callee2);
    }));
  };

  _proto.writeGetService = function writeGetService(_ref4, contextProps) {
    var _this2 = this;

    var service = _ref4.service;
    var serviceBindContext = {};

    if (service) {
      var serviceKeys = Object.keys(service);
      serviceKeys.forEach(function (serviceKey) {
        var serviceIsArray = Array.isArray(service[serviceKey]);

        if (serviceIsArray) {
          serviceBindContext[serviceKey] = _this2.writeServiceBindHandler(service[serviceKey][1], contextProps, serviceBindContext);
        } else {
          serviceBindContext[serviceKey] = _this2.writeServiceBindHandler(service[serviceKey], contextProps, serviceBindContext);
        }
      });
    }

    return serviceBindContext;
  };

  _proto.writeGetView = function writeGetView(_ref5, viewContext) {
    var view = _ref5.view;
    console.log(view, 138);
    var viewBindContext = {};

    if (view) {
      var viewKeys = Object.keys(view);
      viewKeys.forEach(function (viewKey) {
        console.log(viewContext, 142);

        viewBindContext[viewKey] = function () {
          var res = view[viewKey].call(viewContext);
          console.log(res, 146);
          console.log(view[viewKey].toString(), 147);
          return res;
        };
      });
    }

    console.log(viewBindContext, 151);
    return viewBindContext;
  };

  _proto.writeGetViewContext = function writeGetViewContext(controllerBindContext, storeConfig, _ref6) {
    var context = _ref6.context,
        state = _ref6.state,
        refs = _ref6.refs,
        props = _ref6.props,
        _ref6$superContext = _ref6.superContext,
        superContext = _ref6$superContext === void 0 ? null : _ref6$superContext;
    return Object.freeze({
      controller: controllerBindContext,
      state: state,
      refs: refs,
      context: context,
      props: props,
      view: storeConfig.view,
      super: superContext
    });
  };

  _proto.writeGetStoreWithMembrane = function writeGetStoreWithMembrane(storeConfig, store, contextProps) {
    if (storeConfig.membrane) {
      var membraneStore = {};
      contextProps.superContext = store;
      var membraneServiceBindContext = this.writeGetService(storeConfig.membrane, contextProps);
      var membraneControllerBindContext = this.writeGetController(storeConfig.membrane, contextProps, membraneServiceBindContext);
      var membraneViewBindContext = this.writeGetView(storeConfig.membrane, this.writeGetViewContext(membraneControllerBindContext, storeConfig.membrane, contextProps));
      membraneStore.view = _extends({}, store.view, membraneViewBindContext);
      membraneStore.controller = _extends({}, store.controller, membraneControllerBindContext);

      var _this$writeGetUseRedu = this.writeGetUseReducerConfig(storeConfig),
          stateKeys = _this$writeGetUseRedu.stateKeys,
          refKeys = _this$writeGetUseRedu.refKeys;

      return Object.freeze({
        state: pick(store.state, stateKeys),
        refs: pick(store.refs, refKeys),
        controller: pick(membraneStore.controller, Object.keys(storeConfig.controller)),
        view: pick(membraneStore.view, Object.keys(storeConfig.view))
      });
    }

    return Object.freeze(store);
  };

  _proto.writeGetStoreBindContext = function writeGetStoreBindContext(storeConfig, useReducerConfig, contextProps) {
    var store = {};
    var serviceBindContext = this.writeGetService(storeConfig, contextProps);
    var controllerBindContext = this.writeGetController(storeConfig, contextProps, serviceBindContext);
    var viewBindContext = this.writeGetView(storeConfig, this.writeGetViewContext(controllerBindContext, storeConfig, contextProps));
    store.rc = contextProps.rc;
    store.service = serviceBindContext;
    store.state = contextProps.state;
    store.refs = contextProps.refs;
    store.controller = controllerBindContext;
    store.view = viewBindContext;
    store = this.writeGetStoreWithMembrane(storeConfig, store, contextProps, useReducerConfig);
    return store;
  };

  _proto.writeGetRefs = function writeGetRefs(refsKeys, refs) {
    var refContext = {};

    if (refsKeys) {
      refsKeys.forEach(function (refKey) {
        refContext[refKey] = useRef(refs[refKey]);
      });
    }

    return refContext;
  };

  _proto.writeGetRef = function writeGetRef(storeConfig) {
    if (!storeConfig.ref) {
      storeConfig.ref = {};
    }

    if (storeConfig.membrane) {
      if (!storeConfig.membrane.ref) {
        storeConfig.membrane.ref = {};
      }

      return _extends({}, storeConfig.ref, storeConfig.membrane.ref);
    }

    return storeConfig.ref;
  };

  _proto.readPropsHasNoMembrane = function readPropsHasNoMembrane(storeConfig, props) {
    if (!storeConfig.membrane && props) {
      throw new Error(storeConfig.name + " \u6CA1\u6709\u5B9A\u4E49 membrane, \u4F20\u9012\u8FDB\u6765\u7684 props \u662F\u65E0\u6548\u7684");
    }
  };

  _proto.readControllerIsNone = function readControllerIsNone(storeConfig) {
    if (!storeConfig.controller) {
      throw new Error('任何一个 store 都不能没有 controller');
    }
  };

  _proto.main = function main(storeConfig) {
    var _this3 = this;

    this.readControllerIsNone(storeConfig);
    var ref = this.writeGetRef(storeConfig);
    var useReducerConfig = this.writeGetUseReducerConfigWithMembrane(storeConfig);
    var stateKeys = useReducerConfig.stateKeys,
        init = useReducerConfig.init,
        initState = useReducerConfig.initState,
        refKeys = useReducerConfig.refKeys;
    var reducer = reducerUtils.main(stateKeys);
    return function (props) {
      _this3.readPropsHasNoMembrane(storeConfig, props);

      var context = useContext(AppContext);

      var _useReducer = useReducer(reducer, initState, init),
          state = _useReducer[0],
          dispatch = _useReducer[1];

      var rc = createReducerCase.main(stateKeys, dispatch, state);

      var refs = _this3.writeGetRefs(refKeys, ref);

      var store = _this3.writeGetStoreBindContext(storeConfig, useReducerConfig, {
        context: context,
        rc: rc,
        refs: refs,
        state: state,
        props: props
      });

      return store;
    };
  };

  return CreateStoreHook;
}();

export var createStoreHook = new CreateStoreHook();